
LABEL_MAP = {
    'allianceContactList': 'alliance',
    'corporateContactList': 'corp',
    'contactList': 'personal',
}


def parse_contact_list(api_result):
    result = {}
    for rowset in api_result.findall('rowset'):
        contact_list = result[LABEL_MAP[rowset.get('name')]] = {}
        for row in rowset.findall('row'):
            in_watchlist = (row.get('inWatchlist') == 'True'
                            if 'inWatchlist' in row.attrib
                            else None)
            contact_id = int(row.get('contactID'))
            contact_list[contact_id] = {
                'id': contact_id,
                'name': row.get('contactName'),
                'standing': float(row.get('standing')),
                'in_watchlist': in_watchlist
            }

    return result
        



from evelink import api
from evelink import constants

def parse_industry_jobs(api_result):
        rowset = api_result.find('rowset')
        result = {}

        if rowset is None:
            return

        for row in rowset.findall('row'):
            # shortcut to make the following block less painful
            a = row.attrib
            jobID = int(a['jobID'])
            result[jobID] = {
                'line_id': int(a['assemblyLineID']),
                'container_id': int(a['containerID']),
                'input': {
                    'id': int(a['installedItemID']),
                    'blueprint_type': 'copy' if a['installedItemCopy'] == '1' else 'original',
                    'location_id': int(a['installedItemLocationID']),
                    'quantity': int(a['installedItemQuantity']),
                    'prod_level': int(a['installedItemProductivityLevel']),
                    'mat_level': int(a['installedItemMaterialLevel']),
                    'runs_left': int(a['installedItemLicensedProductionRunsRemaining']),
                    'item_flag': int(a['installedItemFlag']),
                    'type_id': int(a['installedItemTypeID']),
                },
                'output': {
                    'location_id': int(a['outputLocationID']),
                    'bpc_runs': int(a['licensedProductionRuns']),
                    'container_location_id': int(a['containerLocationID']),
                    'type_id': int(a['outputTypeID']),
                    'flag': int(a['outputFlag']),
                },
                'runs': int(a['runs']),
                'installer_id': int(a['installerID']),
                'system_id': int(a['installedInSolarSystemID']),
                'multipliers': {
                    'material': float(a['materialMultiplier']),
                    'char_material': float(a['charMaterialMultiplier']),
                    'time': float(a['timeMultiplier']),
                    'char_time': float(a['charTimeMultiplier']),
                },
                'container_type_id': int(a['containerTypeID']),
                'delivered': a['completed'] == '1',
                'finished': a['completedSuccessfully'] == '1',
                'status': constants.Industry.job_status[int(a['completedStatus'])],
                'activity_id': int(a['activityID']),
                'install_ts': api.parse_ts(a['installTime']),
                'begin_ts': api.parse_ts(a['beginProductionTime']),
                'end_ts': api.parse_ts(a['endProductionTime']),
                'pause_ts': api.parse_ts(a['pauseProductionTime']),
            }

        return result



def parse_contract_items(api_result):
    rowset = api_result.find('rowset')
    results = []
    for row in rowset.findall('row'):
        a = row.attrib
        item = {
            'id': int(a['recordID']),
            'type_id': int(a['typeID']),
            'quantity': int(a['quantity']),
            'singleton': a['singleton'] == '1',
            'action': 'offered' if a['included'] == '1' else 'requested',
        }

        # TODO(zigdon): figure out the raw quantity mess.

        results.append(item)

    return results




from evelink import api
from evelink import constants

class Account(object):
    """Wrapper around /account/ of the EVE API.

    Note that a valid API key is required.
    """

    def __init__(self, api):
        self.api = api

    def status(self):
        """Returns the account's subscription status."""
        api_result = self.api.get('account/AccountStatus')

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)

        return {
            'paid_ts': _ts('paidUntil'),
            'create_ts': _ts('createDate'),
            'logins': _int('logonCount'),
            'minutes_played': _int('logonMinutes'),
        }

    def key_info(self):
        """Returns the details of the API key being used to auth."""

        api_result = self.api.get('account/APIKeyInfo')

        key = api_result.find('key')
        result = {
            'access_mask': int(key.attrib['accessMask']),
            'type': constants.APIKey.key_types[key.attrib['type']],
            'expire_ts': api.parse_ts(key.attrib['expires']) if key.attrib['expires'] else None,
            'characters': {},
        }

        rowset = key.find('rowset')
        for row in rowset.findall('row'):
            character = {
                'id': int(row.attrib['characterID']),
                'name': row.attrib['characterName'],
                'corp': {
                    'id': int(row.attrib['corporationID']),
                    'name': row.attrib['corporationName'],
                },
            }
            result['characters'][character['id']] = character

        return result

    def characters(self):
        """Returns all of the characters on an account."""

        api_result = self.api.get('account/Characters')

        rowset = api_result.find('rowset')
        result = {}
        for row in rowset.findall('row'):
            character = {
                'id': int(row.attrib['characterID']),
                'name': row.attrib['name'],
                'corp': {
                    'id': int(row.attrib['corporationID']),
                    'name': row.attrib['corporationName'],
                },
            }
            result[character['id']] = character

        return result



from evelink import api, constants
from evelink.parsing.assets import parse_assets
from evelink.parsing.contact_list import parse_contact_list
from evelink.parsing.contract_bids import parse_contract_bids
from evelink.parsing.contract_items import parse_contract_items
from evelink.parsing.contracts import parse_contracts
from evelink.parsing.industry_jobs import parse_industry_jobs
from evelink.parsing.kills import parse_kills
from evelink.parsing.orders import parse_market_orders
from evelink.parsing.wallet_journal import parse_wallet_journal
from evelink.parsing.wallet_transactions import parse_wallet_transactions

class Char(object):
    """Wrapper around /char/ of the EVE API.

    Note that a valid API key is required.
    """

    def __init__(self, char_id, api):
        self.api = api
        self.char_id = char_id

    def assets(self):
        """Get information about corp assets.

        Each item is a dict, with keys 'id', 'item_type_id',
        'quantity', 'location_id', 'location_flag', and 'packaged'.
        'location_flag' denotes additional information about the
        item's location; see
        http://wiki.eve-id.net/API_Inventory_Flags for more details.

        If the item corresponds to a container, it will have a key
        'contents', which is itself a list of items in the same format
        (potentially recursively holding containers of its own).  If
        the contents do not have 'location_id's of their own, they
        inherit the 'location_id' of their parent container, for
        convenience.

        At the top level, the result is a dict mapping location ID
        (typically a solar system) to a dict containing a 'contents'
        key, which maps to a list of items.  That is, you can think of
        the top-level values as "containers" with no fields except for
        "contents" and "location_id".
        """
        api_result = self.api.get('char/AssetList',
            {'characterID': self.char_id})

        return parse_assets(api_result)

    def contract_bids(self):
        """Lists the latest bids that have been made to any recent auctions."""
        api_result = self.api.get('char/ContractBids',
            {'characterID': self.char_id})

        return parse_contract_bids(api_result)

    def contract_items(self, contract_id):
        """Lists items that a specified contract contains"""
        api_result = self.api.get('char/ContractItems',
            {'characterID': self.char_id, 'contractID': contract_id})

        return parse_contract_items(api_result)

    def contracts(self):
        """Returns a record of all contracts for a specified character"""
        api_result = self.api.get('char/Contracts',
            {'characterID': self.char_id})
        return parse_contracts(api_result)

    def wallet_journal(self, before_id=None, limit=None):
        """Returns a complete record of all wallet activity for a specified character"""
        params = {'characterID': self.char_id}
        if before_id is not None:
            params['fromID'] = before_id
        if limit is not None:
            params['rowCount'] = limit
        api_result = self.api.get('char/WalletJournal', params)

        return parse_wallet_journal(api_result)

    def wallet_info(self):
        """Return a given character's wallet."""
        api_result = self.api.get('char/AccountBalance',
            {'characterID': self.char_id})

        rowset = api_result.find('rowset')
        row = rowset.find('row')
        result = {
            'balance': float(row.attrib['balance']),
            'id': int(row.attrib['accountID']),
            'key': int(row.attrib['accountKey']),
        }
        return result

    def wallet_balance(self):
        """Helper to return just the balance from a given character wallet"""

        return self.wallet_info()['balance']

    def wallet_transactions(self, before_id=None, limit=None):
        """Returns wallet transactions for a character."""

        params = {'characterID': self.char_id}
        if before_id is not None:
            params['fromID'] = before_id
        if limit is not None:
            params['rowCount'] = limit
        api_result = self.api.get('char/WalletTransactions', params)

        return parse_wallet_transactions(api_result)

    def industry_jobs(self):
        """Get a list of jobs for a character"""

        api_result = self.api.get('char/IndustryJobs',
            {'characterID': self.char_id})

        return parse_industry_jobs(api_result)

    def kills(self, before_kill=None):
        """Look up recent kills for a character.

        before_kill:
            Optional. Only show kills before this kill id. (Used for paging.)
        """

        params = {'characterID': self.char_id}
        if before_kill is not None:
            params['beforeKillID'] = before_kill
        api_result = self.api.get('char/KillLog', params)

        return parse_kills(api_result)

    def notifications(self):
        """Returns the message headers for notifications."""
        api_result = self.api.get('char/Notifications',
            {'characterID': self.char_id})

        result = {}
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            notification_id = int(a['notificationID'])
            result[notification_id] = {
                'id': notification_id,
                'type_id': a['typeID'],
                'sender_id': int(a['senderID']),
                'timestamp': api.parse_ts(a['sentDate']),
                'read': a['read'] == '1',
            }

        return result

    def notification_texts(self, notification_ids):
        """Returns the message bodies for notifications."""
        api_result = self.api.get('char/NotificationTexts',
            {'characterID': self.char_id, 'IDs': notification_ids})

        result = {}
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            notification_id = int(row.attrib['notificationID'])
            notification = {'id': notification_id}
            notification.update(api.parse_keyval_data(row.text))
            result[notification_id] = notification

        missing_ids = api_result.find('missingIDs')
        if missing_ids is not None:
            for missing_id in missing_ids.text.split(","):
                result[missing_id] = None

        return result

    def standings(self):
        """Returns the standings towards a character from NPC entities."""
        api_result = self.api.get('char/Standings',
            {'characterID': self.char_id})

        result = {}
        rowsets = {}
        for rowset in api_result.find('characterNPCStandings').findall('rowset'):
            rowsets[rowset.attrib['name']] = rowset

        _name_map = {
            'agents': 'agents',
            'corps': 'NPCCorporations',
            'factions': 'factions',
        }

        for key, rowset_name in _name_map.iteritems():
            result[key] = {}
            for row in rowsets[rowset_name].findall('row'):
                a = row.attrib
                from_id = int(a['fromID'])
                result[key][from_id] = {
                    'id': from_id,
                    'name': a['fromName'],
                    'standing': float(a['standing']),
                }

        return result

    def character_sheet(self):
        """Returns attributes relating to a specific character."""
        api_result = self.api.get('char/CharacterSheet',
            {'characterID': self.char_id})

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)
        result = {
            'id': _int('characterID'),
            'name': _str('name'),
            'create_ts': _ts('DoB'),
            'race': _str('race'),
            'bloodline': _str('bloodLine'),
            'ancestry': _str('ancestry'),
            'gender': _str('gender'),
            'corp': {
                'id': _int('corporationID'),
                'name': _str('corporationName'),
            },
            'alliance': {
                'id': _int('allianceID') or None,
                'name': _str('allianceName'),
            },
            'clone': {
                'name': _str('cloneName'),
                'skillpoints': _int('cloneSkillPoints'),
            },
            'balance': _float('balance'),
            'attributes': {},
        }

        for attr in ('intelligence', 'memory', 'charisma', 'perception', 'willpower'):
            result['attributes'][attr] = {}
            base = int(api_result.findtext('attributes/%s' % attr))
            result['attributes'][attr]['base'] = base
            result['attributes'][attr]['total'] = base
            bonus = api_result.find('attributeEnhancers/%sBonus' % attr)
            if bonus is not None:
                mod = int(bonus.findtext('augmentatorValue'))
                result['attributes'][attr]['total'] += mod
                result['attributes'][attr]['bonus'] = {
                    'name': bonus.findtext('augmentatorName'),
                    'value': mod,
                }

        rowsets = {}
        for rowset in api_result.findall('rowset'):
            key = rowset.attrib['name']
            rowsets[key] = rowset

        result['skills'] = []
        result['skillpoints'] = 0
        for skill in rowsets['skills']:
            a = skill.attrib
            sp = int(a['skillpoints'])
            result['skills'].append({
                'id': int(a['typeID']),
                'skillpoints': sp,
                'level': int(a['level']),
                'published': a['published'] == '1',
            })
            result['skillpoints'] += sp

        result['certificates'] = set()
        for cert in rowsets['certificates']:
            result['certificates'].add(int(cert.attrib['certificateID']))

        result['roles'] = {}
        for our_role, ccp_role in constants.Char().corp_roles.iteritems():
            result['roles'][our_role] = {}
            for role in rowsets[ccp_role]:
                a = role.attrib
                role_id = int(a['roleID'])
                result['roles'][our_role][role_id] = {
                    'id': role_id,
                    'name': a['roleName'],
                }

        result['titles'] = {}
        for title in rowsets['corporationTitles']:
            a = title.attrib
            title_id = int(a['titleID'])
            result['titles'][title_id] = {
                'id': title_id,
                'name': a['titleName'],
            }

        return result

    def contacts(self):
        """Return a character's personal, corp and alliance contact lists."""
        api_result = self.api.get('char/ContactList',
            {'characterID': self.char_id})

        return parse_contact_list(api_result)

    def orders(self):
        """Return a given character's buy and sell orders."""
        api_result = self.api.get('char/MarketOrders',
            {'characterID': self.char_id})

        return parse_market_orders(api_result)

    def research(self):
        """Returns information about the agents with whom the character is doing research."""

        api_result = self.api.get('char/Research',
            {'characterID': self.char_id})

        rowset = api_result.find('rowset')
        rows = rowset.findall('row')
        result = {}
        for row in rows:
            a = row.attrib
            id = int(a['agentID'])
            result[id] = {
                'id': id,
                'skill_id': int(a['skillTypeID']),
                'timestamp': api.parse_ts(a['researchStartDate']),
                'per_day': float(a['pointsPerDay']),
                'remaining': float(a['remainderPoints']),
            }

        return result

    def current_training(self):
        """Returns the skill that is currently being trained by a specified character"""

        api_result = self.api.get('char/SkillInTraining',
            {'characterID': self.char_id})

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)
        result = {
            'start_ts': _ts('trainingStartTime'),
            'end_ts': _ts('trainingEndTime'),
            'type_id': _int('trainingTypeID'),
            'start_sp': _int('trainingStartSP'),
            'end_sp': _int('trainingDestinationSP'),
            'current_ts': _ts('currentTQTime'),
            'level': _int('trainingToLevel'),
            'active': _bool('skillInTraining'),
        }

        return result

    def skill_queue(self):
        """returns the skill queue of the character"""
        api_result = self.api.get('char/SkillQueue',
            {'characterID': self.char_id})

        rowset = api_result.find('rowset')
        rows = rowset.findall('row')
        result = []
        for row in rows:
            a = row.attrib
            line = {
                'position': int(a['queuePosition']),
                'type_id': int(a['typeID']),
                'level': int(a['level']),
                'start_sp': int(a['startSP']),
                'end_sp': int(a['endSP']),
                'start_ts': api.parse_ts(a['startTime']),
                'end_ts': api.parse_ts(a['endTime']),
            }

            result.append(line)

        return result

    def messages(self):
        """Returns a list of headers for a character's mail."""
        api_result = self.api.get('char/MailMessages',
            {'characterID': self.char_id})

        rowset = api_result.find('rowset')
        results = []
        for row in rowset.findall('row'):
            a = row.attrib
            message = {
                'id': int(a['messageID']),
                'sender_id': int(a['senderID']),
                'timestamp': api.parse_ts(a['sentDate']),
                'title': a['title'],
                'to': {},
            }

            org_id = a['toCorpOrAllianceID']
            message['to']['org_id'] = int(org_id) if org_id else None

            char_ids = a['toCharacterIDs']
            message['to']['char_ids'] = [int(i) for i in char_ids.split(',')] if char_ids else None

            list_ids = a['toListID']
            message['to']['list_ids'] = [int(i) for i in list_ids.split(',')] if list_ids else None

            results.append(message)

        return results

    def message_bodies(self, message_ids):
        """Returns the actual body content of a set of mail messages.

        NOTE: You *must* have recently looked up the headers of
        any messages you are requesting bodies for (via the 'messages'
        method) or else this call will fail.
        """
        api_result = self.api.get('char/MailBodies',
            {'characterID': self.char_id, 'ids': message_ids})

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            message_id = int(row.attrib['messageID'])
            results[message_id] = row.text

        missing_set = api_result.find('missingMessageIDs')
        if missing_set is not None:
            missing_ids = [int(i) for i in missing_set.text.split(',')]
            for missing_id in missing_ids:
                results[missing_id] = None

        return results

    def mailing_lists(self):
        """Returns the mailing lists to which a character is subscribed."""
        api_result = self.api.get('char/MailingLists')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            results[int(a['listID'])] = a['displayName']

        return results

    def calendar_events(self):
        """Returns the list of upcoming calendar events for a character."""
        api_result = self.api.get('char/UpcomingCalendarEvents',
            {'characterID': self.char_id})

        results = {}
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            event = {
                'id': int(a['eventID']),
                'owner': {
                    'id': int(a['ownerID']),
                    'name': a['ownerName'] or None,
                },
                'start_ts': api.parse_ts(a['eventDate']),
                'title': a['eventTitle'],
                'duration': int(a['duration']),
                'important': a['importance'] == '1',
                'description': a['eventText'],
                'response': a['response'],
            }
            results[event['id']] = event

        return results

    def calendar_attendees(self, event_ids):
        """Returns the list of attendees for the specified calendar event.

        This function takes a list of event IDs and returns a dict of dicts,
        with the top-level dict being keyed by event ID and the children
        keyed by the character IDs of the attendees.

        NOTE: You must have recently fetched the list of calendar events
        (using the 'calendar_events' method) before calling this method.
        """
        api_result = self.api.get('char/CalendarEventAttendees',
            {'characterID': self.char_id, 'eventIDs': event_ids})

        results = dict((int(i),{}) for i in event_ids)
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            attendee = {
                'id': int(a['characterID']),
                'name': a['characterName'],
                'response': a['response'],
            }
            results[int(a['eventID'])][attendee['id']] = attendee

        return results

    def event_attendees(self, event_id):
        """Returns the attendees for a single event.

        (This is a convenience wrapper around 'calendar_attendees'.)

        NOTE: You must have recently fetched the list of calendar events
        (using the 'calendar_events' method) before calling this method.
        """
        return self.calendar_attendees([event_id])[int(event_id)]

    def faction_warfare_stats(self):
        """Returns FW stats for this character, if enrolled in FW.

        NOTE: This will return an error instead if the character
        is not enrolled in Faction Warfare.
        """
        api_result = self.api.get('char/FacWarStats',
            {'characterID': self.char_id})

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)

        return {
            'faction': {
                'id': _int('factionID'),
                'name': _str('factionName'),
            },
            'enlist_ts': _ts('enlisted'),
            'rank': {
                'current': _int('currentRank'),
                'highest': _int('highestRank'),
            },
            'kills': {
                'yesterday': _int('killsYesterday'),
                'week': _int('killsLastWeek'),
                'total': _int('killsTotal'),
            },
            'points': {
                'yesterday': _int('victoryPointsYesterday'),
                'week': _int('victoryPointsLastWeek'),
                'total': _int('victoryPointsTotal'),
            },
        }

    def medals(self):
        """Returns a list of medals the character has."""

        api_result = self.api.get('char/Medals',
            {'characterID': self.char_id})

        result = {'current': {}, 'other': {}}
        _map = {
            'currentCorporation': 'current',
            'otherCorporations': 'other',
        }

        for rowset in api_result.findall('rowset'):
            name = _map[rowset.attrib['name']]
            for row in rowset.findall('row'):
                a = row.attrib
                medal_id = int(a['medalID'])
                result[name][medal_id] = {
                    'id': medal_id,
                    'reason': a['reason'],
                    'public': a['status'] == 'public',
                    'issuer_id': int(a['issuerID']),
                    'corp_id': int(a['corporationID']),
                    'title': a['title'],
                    'description': a['description'],
                }

        return result

    def contact_notifications(self):
        """Returns pending contact notifications."""
        api_result = self.api.get('char/ContactNotifications',
            {'characterID': self.char_id})

        results = {}
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            note = {
                'id': int(a['notificationID']),
                'sender': {
                    'id': int(a['senderID']),
                    'name': a['senderName'],
                },
                'timestamp': api.parse_ts(a['sentDate']),
                'data': api.parse_keyval_data(a['messageData']),
            }
            results[note['id']] = note

        return results


# vim: set ts=4 sts=4 sw=4 et:



import calendar
import functools
import logging
import re
import time
from urllib import urlencode
import urllib2
from xml.etree import ElementTree

_log = logging.getLogger('evelink.api')

def _clean(v):
    """Convert parameters into an acceptable format for the API."""
    if isinstance(v, (list, set, tuple)):
        return ",".join(str(i) for i in v)
    else:
        return str(v)


def parse_ts(v):
    """Parse a timestamp from EVE API XML into a unix-ish timestamp."""
    if v == '':
        return None
    ts = calendar.timegm(time.strptime(v, "%Y-%m-%d %H:%M:%S"))
    # Deal with EVE's nonexistent 0001-01-01 00:00:00 timestamp
    return ts if ts > 0 else None


def get_named_value(elem, field):
    """Returns the string value of the named child element."""
    try:
        return elem.find(field).text
    except AttributeError:
        return None


def get_ts_value(elem, field):
    """Returns the timestamp value of the named child element."""
    val = get_named_value(elem, field)
    if val:
        return parse_ts(val)
    return None


def get_int_value(elem, field):
    """Returns the integer value of the named child element."""
    val = get_named_value(elem, field)
    if val:
        return int(val)
    return val


def get_float_value(elem, field):
    """Returns the float value of the named child element."""
    val = get_named_value(elem, field)
    if val:
        return float(val)
    return val


def get_bool_value(elem, field):
    """Returns the boolean value of the named child element."""
    val = get_named_value(elem, field)
    if val == 'True':
        return True
    elif val == 'False':
        return False
    return None


def elem_getters(elem):
    """Returns a tuple of (_str, _int, _float, _bool, _ts) functions.

    These are getters closed around the provided element.
    """
    _str = lambda key: get_named_value(elem, key)
    _int = lambda key: get_int_value(elem, key)
    _float = lambda key: get_float_value(elem, key)
    _bool = lambda key: get_bool_value(elem, key)
    _ts = lambda key: get_ts_value(elem, key)

    return _str, _int, _float, _bool, _ts


def parse_keyval_data(data_string):
    """Parse 'key: value' lines from a LF-delimited string."""
    keyval_pairs = data_string.strip().split('\n')
    results = {}
    for pair in keyval_pairs:
        key, _, val = pair.strip().partition(': ')

        if 'Date' in key:
            val = parse_ms_date(val)
        elif val == 'null':
            val = None
        elif re.match(r"^-?\d+$", val):
            val = int(val)
        elif re.match(r"-?\d+\.\d+", val):
            val = float(val)

        results[key] = val
    return results

def parse_ms_date(date_string):
    """Convert MS date format into epoch"""

    return int(date_string)/10000000 - 11644473600;

class APIError(Exception):
    """Exception raised when the EVE API returns an error."""

    def __init__(self, code=None, message=None):
        self.code = code
        self.message = message

    def __repr__(self):
        return "APIError(%r, %r)" % (self.code, self.message)

    def __str__(self):
        return "%s (code=%d)" % (self.message, int(self.code))

class APICache(object):
    """Minimal interface for caching API requests.

    This very basic implementation simply stores values in
    memory, with no other persistence. You can subclass it
    to define a more complex/featureful/persistent cache.
    """

    def __init__(self):
        self.cache = {}

    def get(self, key):
        """Return the value referred to by 'key' if it is cached.

        key:
            a result from the Python hash() function.
        """
        result = self.cache.get(key)
        if not result:
            return None
        value, expiration = result
        if expiration < time.time():
            del self.cache[key]
            return None
        return value

    def put(self, key, value, duration):
        """Cache the provided value, referenced by 'key', for the given duration.

        key:
            a result from the Python hash() function.
        value:
            an xml.etree.ElementTree.Element object
        duration:
            a number of seconds before this cache entry should expire.
        """
        expiration = time.time() + duration
        self.cache[key] = (value, expiration)


class API(object):
    """A wrapper around the EVE API."""

    def __init__(self, base_url="api.eveonline.com", cache=None, api_key=None):
        self.base_url = base_url

        cache = cache or APICache()
        if not isinstance(cache, APICache):
            raise ValueError("The provided cache must subclass from APICache.")
        self.cache = cache

        if api_key and len(api_key) != 2:
            raise ValueError("The provided API key must be a tuple of (keyID, vCode).")
        self.api_key = api_key

    def _cache_key(self, path, params):
        sorted_params = sorted(params.iteritems())
        # Paradoxically, Shelve doesn't like integer keys.
        return str(hash((path, tuple(sorted_params))))

    def get(self, path, params=None):
        """Request a specific path from the EVE API.

        The supplied path should be a slash-separated path
        frament, e.g. "corp/AssetList". (Basically, the portion
        of the API url in between the root / and the .xml bit.)
        """

        params = params or {}
        params = dict((k, _clean(v)) for k,v in params.iteritems())

        _log.debug("Calling %s with params=%r", path, params)
        if self.api_key:
            _log.debug("keyID and vCode added")
            params['keyID'] = self.api_key[0]
            params['vCode'] = self.api_key[1]

        key = self._cache_key(path, params)
        cached_result = self.cache.get(key)
        if cached_result is not None:
            # Cached APIErrors should be re-raised
            if isinstance(cached_result, APIError):
                _log.error("Raising cached error: %r" % cached_result)
                raise cached_result
            # Normal cached results get returned
            _log.debug("Cache hit, returning cached payload")
            return cached_result

        params = urlencode(params)

        full_path = "https://%s/%s.xml.aspx" % (self.base_url, path)

        response = self.send_request(full_path, params)

        tree = ElementTree.parse(response)

        current_time = get_ts_value(tree, 'currentTime')
        expires_time = get_ts_value(tree, 'cachedUntil')

        error = tree.find('error')
        if error is not None:
            code = error.attrib['code']
            message = error.text.strip()
            exc = APIError(code, message)

            self.cache.put(key, exc, expires_time - current_time)
            _log.error("Raising API error: %r" % exc)
            raise exc

        result = tree.find('result')

        self.cache.put(key, result, expires_time - current_time)
        return result

    def send_request(self, full_path, params):
        try:
            if params:
                # POST request
                _log.debug("POSTing request")
                return urllib2.urlopen(full_path, params)
            else:
                # GET request
                _log.debug("GETting request")
                return urllib2.urlopen(full_path)
        except urllib2.URLError as e:
            # TODO: Handle this better?
            raise e

def auto_api(func):
    """A decorator to automatically provide an API instance.

    Functions decorated with this will have the api= kwarg
    automatically supplied with a default-initialized API()
    object if no other API object is supplied.
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if 'api' not in kwargs:
            kwargs['api'] = API()
        return func(*args, **kwargs)
    return wrapper


# vim: set ts=4 sts=4 sw=4 et:



import datetime
import json
import urllib
from xml.etree import ElementTree

try:
    import urllib2
except ImportError:
    urllib2 = None

class EVECentral(object):

    def __init__(self, url_fetch_func=None,
        api_base='http://api.eve-central.com/api'):
        super(EVECentral, self).__init__()

        self.api_base = api_base

        if url_fetch_func is not None:
            self.url_fetch = url_fetch_func
        elif urllib2 is not None:
            self.url_fetch = self._default_fetch_func
        else:
            raise ValueError("urllib2 not available - specify url_fetch_func")

    def _default_fetch_func(self, url):
        """Fetches a given URL using GET and returns the response."""
        return urllib2.urlopen(url).read()

    def market_stats(self, type_ids, hours=24, regions=None, system=None,
        quantity_threshold=None):
        """Fetches market statistics for one or more items.

        Optional filters:
            hours (int) - Time period to compute statistics for.
            regions (list of ints) - Region id(s) for which to compute stats.
            systems (int) - System id for which to compute stats.
            quantity_threshold (int) - minimum size of order to consider.
        """

        params = [('typeid', type_ids), ('hours', hours)]
        if regions:
            params.append(('regionlimit', regions))
        if system:
            params.append(('usesystem', system))
        if quantity_threshold:
            params.append(('minQ', quantity_threshold))

        query = urllib.urlencode(params, True)
        url = '%s/marketstat?%s' % (self.api_base, query)

        response = self.url_fetch(url)
        api_result = ElementTree.fromstring(response)

        results = {}
        stats = api_result.find('marketstat')
        for type_section in stats.findall('type'):
            type_id = int(type_section.attrib['id'])
            type_result = {'id': type_id}
            for sub in ('all', 'buy', 'sell'):
                s = type_section.find(sub)
                sub_result = {
                    'volume': int(s.find('volume').text),
                    'avg': float(s.find('avg').text),
                    'max': float(s.find('max').text),
                    'min': float(s.find('min').text),
                    'stddev': float(s.find('stddev').text),
                    'median': float(s.find('median').text),
                    'percentile': float(s.find('percentile').text),
                }
                type_result[sub] = sub_result
            results[type_id] = type_result

        return results

    def item_market_stats(self, type_id, *args, **kwargs):
        """Fetch market statistics for a single item.

        (Convenience wrapper for market_stats.)
        """
        return self.market_stats([type_id], *args, **kwargs)[int(type_id)]

    def item_orders(self, type_id, hours=360, regions=None, system=None,
        quantity_threshold=None):
        """Fetches market orders for a given item.

        Optional filters:
            hours (int) - The time period from which to fetch posted orders.
            regions (list of ints) - Region id(s) for which to fetch orders.
            systems (int) - System id for which to fetch orders.
            quantity_threshold (int) - minimum size of order to consider.
        """

        params = [('typeid', type_id), ('sethours', hours)]
        if regions:
            params.append(('regionlimit', regions))
        if system:
            params.append(('usesystem', system))
        if quantity_threshold:
            params.append(('setminQ', quantity_threshold))

        query = urllib.urlencode(params, True)
        url = '%s/quicklook?%s' % (self.api_base, query)

        response = self.url_fetch(url)
        return self._parse_item_orders(response)

    def item_orders_on_route(self, type_id, start, dest, hours=360,
        quantity_threshold=None):
        """Fetches market orders for a given item along a shortest-path route.

        Optional filters:
            hours (int) - The time period from which to fetch posted orders.
            quantity_threshold (int) - minimum size of order to consider.
        """

        params = [('sethours', hours)]
        if quantity_threshold:
            params.append(('setminQ', quantity_threshold))

        query = urllib.urlencode(params, True)
        url = '%s/quicklook/onpath/from/%s/to/%s/fortype/%s?%s' % (
            self.api_base, start, dest, type_id, query)

        response = self.url_fetch(url)
        return self._parse_item_orders(response)

    def _parse_item_orders(self, response):
        """Shared parsing functionality for market order data from EVE-Central."""
        api_result = ElementTree.fromstring(response)

        res = api_result.find('quicklook')
        regions = res.find('regions').findall('region')
        results = {
            'id': int(res.find('item').text),
            'name': res.find('itemname').text,
            'hours': int(res.find('hours').text),
            'quantity_min': int(res.find('minqty').text),
            'regions': [r.text for r in regions] or None,
            'orders': {},
        }

        for act in ('buy', 'sell'):
            sub_result = {}
            for order in res.find('%s_orders' % act).findall('order'):
                order_id = int(order.attrib['id'])
                o = {
                    'id': order_id,
                    'region_id': int(order.find('region').text),
                    'station': {
                        'id': int(order.find('station').text),
                        'name': order.find('station_name').text,
                    },
                    'security': float(order.find('security').text),
                    'range': int(order.find('range').text),
                    'price': float(order.find('price').text),
                    'volume': {
                        'remaining': int(order.find('vol_remain').text),
                        'minimum': int(order.find('min_volume').text),
                    },
                    'expires': datetime.datetime.strptime(
                        order.find('expires').text,
                        "%Y-%m-%d",
                    ).date(),
                    'reported': datetime.datetime.strptime(
                        order.find('reported_time').text,
                        "%m-%d %H:%M:%S",
                    ),
                }

                # Correct errors due to EVE-Central only reporting the month
                # and day of the report, not the year. (Assumes reports are
                # never from the future and never older than a year.)
                this_year = datetime.datetime.now().year
                o['reported'] = o['reported'].replace(year=this_year)
                if o['reported'] > datetime.datetime.now():
                    previous_year = o['reported'].year - 1
                    o['reported'].replace(year=previous_year)

                sub_result[order_id] = o

            results['orders'][act] = sub_result

        return results

    def route(self, start, dest):
        """Returns a shortest-path route between two systems.

        Both start and dest can be either exact system names or
        system IDs.
        """

        url = '%s/route/from/%s/to/%s' % (self.api_base, start, dest)
        response = self.url_fetch(url)

        stops = json.loads(response)

        results = []
        for stop in stops:
            results.append({
                'from': {
                    'id': stop['fromid'],
                    'name': stop['from'],
                },
                'to': {
                    'id': stop['toid'],
                    'name': stop['to'],
                },
                'security_change': stop['secchange'],
            })

        return results



# vim: set et ts=4 sts=4 sw=4:



from evelink import api
from evelink import constants
import time

def parse_contracts(api_result):
    rowset = api_result.find('rowset')
    if rowset is None:
        return

    results = {}
    for row in rowset.findall('row'):
        a = row.attrib
        contract = {
            'id': int(a['contractID']),
            'issuer': int(a['issuerID']),
            'issuer_corp': int(a['issuerCorpID']),
            'assignee': int(a['assigneeID']),
            'acceptor': int(a['acceptorID']),
            'start': int(a['startStationID']),
            'end': int(a['endStationID']),
            'type': a['type'],
            'status': a['status'],
            'corp': a['forCorp'] == '1',
            'availability': a['availability'],
            'issued': api.parse_ts(a['dateIssued']),
            'days': int(a['numDays']),
            'price': float(a['price']),
            'reward': float(a['reward']),
            'collateral': float(a['collateral']),
            'buyout': float(a['buyout']),
            'volume': float(a['volume']),
            'title': a['title']
        }
        contract['expired'] = api.parse_ts(a['dateExpired'])
        contract['accepted'] = api.parse_ts(a['dateAccepted'])
        contract['completed'] = api.parse_ts(a['dateCompleted'])
        results[contract['id']] = contract
    return results



from evelink import api, constants
from evelink.parsing.assets import parse_assets
from evelink.parsing.contact_list import parse_contact_list
from evelink.parsing.contract_bids import parse_contract_bids
from evelink.parsing.contract_items import parse_contract_items
from evelink.parsing.contracts import parse_contracts
from evelink.parsing.industry_jobs import parse_industry_jobs
from evelink.parsing.kills import parse_kills
from evelink.parsing.orders import parse_market_orders
from evelink.parsing.wallet_journal import parse_wallet_journal
from evelink.parsing.wallet_transactions import parse_wallet_transactions

class Corp(object):
    """Wrapper around /corp/ of the EVE API.

    Note that a valid corp API key is required.
    """

    def __init__(self, api):
        self.api = api

    def corporation_sheet(self, corp_id=None):
        """Get information about a corporation.

        NOTE: This method may be called with or without specifying
        a corporation ID. If a corporation ID is specified, the public
        information for that corporation will be returned, and no api
        key is necessary. If a corporation ID is *not* specified,
        a corp api key *must* be provided, and the private information
        for that corporation will be returned along with the public info.
        """
        params = {}
        if corp_id is not None:
            params['corporationID'] = corp_id

        api_result = self.api.get("corp/CorporationSheet", params)

        def get_logo_details(logo_result):
            _str, _int, _float, _bool, _ts = api.elem_getters(logo_result)
            return {
                'graphic_id': _int('graphicID'),
                'shapes': [
                    {'id': _int('shape1'), 'color': _int('color1')},
                    {'id': _int('shape2'), 'color': _int('color2')},
                    {'id': _int('shape3'), 'color': _int('color3')},
                ],
            }

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)

        result = {
            'id': _int('corporationID'),
            'name': _str('corporationName'),
            'ticker': _str('ticker'),
            'ceo': {
                'id': _int('ceoID'),
                'name': _str('ceoName'),
            },
            'hq': {
                'id': _int('stationID'),
                'name': _str('stationName'),
            },
            'description': _str('description'),
            'url': _str('url'),
            'alliance': {
                'id': _int('allianceID') or None,
                'name': _str('allianceName') or None,
            },
            'tax_percent': _float('taxRate'),
            'members': {
                'current': _int('memberCount'),
            },
            'shares': _int('shares'),
            'logo': get_logo_details(api_result.find('logo')),
        }

        if corp_id is None:
            result['members']['limit'] = _int('memberLimit')

            rowsets = dict((r.attrib['name'], r) for r in api_result.findall('rowset'))

            division_types = {
                'hangars': 'divisions',
                'wallets': 'walletDivisions',
            }

            for key, rowset_name in division_types.iteritems():
                divisions = {}
                for row in rowsets[rowset_name].findall('row'):
                    a = row.attrib
                    divisions[int(a['accountKey'])] = a['description']

                result[key] = divisions

        return result

    def industry_jobs(self):
        """Get a list of jobs for a corporation."""

        api_result = self.api.get('corp/IndustryJobs')

        return parse_industry_jobs(api_result)

    def npc_standings(self):
        """Returns information about the corporation's standings towards NPCs.

        NOTE: This is *only* NPC standings. Player standings are accessed
        via the 'contacts' method.
        """
        api_result = self.api.get('corp/Standings')
        container = api_result.find('corporationNPCStandings')

        rowsets = dict((r.attrib['name'], r) for r in container.findall('rowset'))
        results = {
            'agents': {},
            'corps': {},
            'factions': {},
        }

        _standing_types = {
            'agents': 'agents',
            'corps': 'NPCCorporations',
            'factions': 'factions',
        }

        for key, rowset_name in _standing_types.iteritems():
            for row in rowsets[rowset_name].findall('row'):
                a = row.attrib
                standing = {
                    'id': int(a['fromID']),
                    'name': a['fromName'],
                    'standing': float(a['standing']),
                }
                results[key][standing['id']] = standing

        return results

    def kills(self, before_kill=None):
        """Look up recent kills for a corporation.

        before_kill:
            Optional. Only show kills before this kill id. (Used for paging.)
        """

        params = {}
        if before_kill is not None:
            params['beforeKillID'] = before_kill
        api_result = self.api.get('corp/KillLog', params)

        return parse_kills(api_result)

    def wallet_info(self):
        """Get information about corp wallets."""

        api_result = self.api.get('corp/AccountBalance')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            wallet = {
                'balance': float(row.attrib['balance']),
                'id': int(row.attrib['accountID']),
                'key': int(row.attrib['accountKey']),
            }
            results[wallet['key']] = wallet

        return results

    def wallet_journal(self, before_id=None, limit=None):
        """Returns wallet journal for a corporation."""

        params = {}
        if before_id is not None:
            params['fromID'] = before_id
        if limit is not None:
            params['rowCount'] = limit
        api_result = self.api.get('corp/WalletJournal', params)

        return parse_wallet_journal(api_result)

    def wallet_transactions(self, before_id=None, limit=None):
        """Returns wallet transactions for a corporation."""

        params = {}
        if before_id is not None:
            params['fromID'] = before_id
        if limit is not None:
            params['rowCount'] = limit
        api_result = self.api.get('corp/WalletTransactions', params)

        return parse_wallet_transactions(api_result)

    def orders(self):
        """Return a corporation's buy and sell orders."""
        api_result = self.api.get('corp/MarketOrders')

        return parse_market_orders(api_result)

    def assets(self):
        """Get information about corp assets.

        Each item is a dict, with keys 'id', 'item_type_id',
        'quantity', 'location_id', 'location_flag', and 'packaged'.
        'location_flag' denotes additional information about the
        item's location; see
        http://wiki.eve-id.net/API_Inventory_Flags for more details.

        If the item corresponds to a container, it will have a key
        'contents', which is itself a list of items in the same format
        (potentially recursively holding containers of its own).  If
        the contents do not have 'location_id's of their own, they
        inherit the 'location_id' of their parent container, for
        convenience.

        At the top level, the result is a dict mapping location ID
        (typically a solar system) to a dict containing a 'contents'
        key, which maps to a list of items.  That is, you can think of
        the top-level values as "containers" with no fields except for
        "contents" and "location_id".
        """
        api_result = self.api.get('corp/AssetList')
        return parse_assets(api_result)

    def faction_warfare_stats(self):
        """Returns stats from faction warfare if this corp is enrolled.

        NOTE: This will raise an APIError if the corp is not enrolled in
        Faction Warfare.
        """
        api_result = self.api.get('corp/FacWarStats')

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)

        return {
            'faction': {
                'id': _int('factionID'),
                'name': _str('factionName'),
            },
            'start_ts': _ts('enlisted'),
            'pilots': _int('pilots'),
            'kills': {
                'yesterday': _int('killsYesterday'),
                'week': _int('killsLastWeek'),
                'total': _int('killsTotal'),
            },
            'points': {
                'yesterday': _int('victoryPointsYesterday'),
                'week': _int('victoryPointsLastWeek'),
                'total': _int('victoryPointsTotal'),
            },
        }

    def contract_bids(self):
        """Lists the latest bids that have been made to any recent auctions."""
        api_result = self.api.get('corp/ContractBids')

        return parse_contract_bids(api_result)

    def contract_items(self, contract_id):
        """Lists items that a specified contract contains"""
        api_result = self.api.get('corp/ContractItems',
            {'contractID': contract_id})

        return parse_contract_items(api_result)

    def contracts(self):
        """Get information about corp contracts."""
        api_result = self.api.get('corp/Contracts')
        return parse_contracts(api_result)

    def shareholders(self):
        """Get information about a corp's shareholders."""
        api_result = self.api.get('corp/Shareholders')

        results = {
            'char': {},
            'corp': {},
        }
        rowsets = dict((r.attrib['name'], r) for r in api_result.findall('rowset'))

        for row in rowsets['characters'].findall('row'):
            a = row.attrib
            holder = {
                'id': int(a['shareholderID']),
                'name': a['shareholderName'],
                'corp': {
                    'id': int(a['shareholderCorporationID']),
                    'name': a['shareholderCorporationName'],
                },
                'shares': int(a['shares']),
            }
            results['char'][holder['id']] = holder

        for row in rowsets['corporations'].findall('row'):
            a = row.attrib
            holder = {
                'id': int(a['shareholderID']),
                'name': a['shareholderName'],
                'shares': int(a['shares']),
            }
            results['corp'][holder['id']] = holder

        return results

    def contacts(self):
        """Return the corp's corp and alliance contact lists."""
        api_result = self.api.get('corp/ContactList')
        return parse_contact_list(api_result)

    def titles(self):
        """Returns information about the corporation's titles."""
        api_result = self.api.get('corp/Titles')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            title = {
                'id': int(a['titleID']),
                'name': a['titleName'],
                'roles': {},
                'can_grant': {},
            }
            rowsets = dict((r.attrib['name'], r) for r in row.findall('rowset'))

            def get_roles(rowset_name):
                roles = {}
                for role_row in rowsets[rowset_name].findall('row'):
                    ra = role_row.attrib
                    role = {
                        'id': int(ra['roleID']),
                        'name': ra['roleName'],
                        'description': ra['roleDescription'],
                    }
                    roles[role['id']] = role
                return roles

            for key, rowset_name in constants.Corp.role_types.iteritems():
                roles = get_roles(rowset_name)
                title['roles'][key] = roles

            for key, rowset_name in constants.Corp.grantable_types.iteritems():
                roles = get_roles(rowset_name)
                title['can_grant'][key] = roles

            results[title['id']] = title

        return results

    def starbases(self):
        """Returns information about the corporation's POSes."""
        api_result = self.api.get('corp/StarbaseList')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            starbase = {
                'id': int(a['itemID']),
                'type_id': int(a['typeID']),
                'location_id': int(a['locationID']),
                'moon_id': int(a['moonID']),
                'state': constants.Corp.pos_states[int(a['state'])],
                'state_ts': api.parse_ts(a['stateTimestamp']),
                'online_ts': api.parse_ts(a['onlineTimestamp']),
                'standings_owner_id': int(a['standingOwnerID']),
            }
            results[starbase['id']] = starbase

        return results

    def starbase_details(self, starbase_id):
        """Returns details about the specified POS."""
        api_result = self.api.get('corp/StarbaseDetail', {'itemID': starbase_id})

        _str, _int, _float, _bool, _ts = api.elem_getters(api_result)

        general_settings = api_result.find('generalSettings')
        combat_settings = api_result.find('combatSettings')

        def get_fuel_bay_perms(settings):
            # Two 2-bit fields
            usage_flags = int(settings.find('usageFlags').text)
            take_value = usage_flags % 4
            view_value = (usage_flags >> 2) % 4
            return {
                'view': constants.Corp.pos_permission_entities[view_value],
                'take': constants.Corp.pos_permission_entities[take_value],
            }

        def get_deploy_perms(settings):
            # Four 2-bit fields
            deploy_flags = int(settings.find('deployFlags').text)
            anchor_value = (deploy_flags >> 6) % 4
            unanchor_value = (deploy_flags >> 4) % 4
            online_value = (deploy_flags >> 2) % 4
            offline_value = deploy_flags % 4
            return {
                'anchor': constants.Corp.pos_permission_entities[anchor_value],
                'unanchor': constants.Corp.pos_permission_entities[unanchor_value],
                'online': constants.Corp.pos_permission_entities[online_value],
                'offline': constants.Corp.pos_permission_entities[offline_value],
            }

        def get_combat_settings(settings):
            result = {
                'standings_owner_id': int(settings.find('useStandingsFrom').attrib['ownerID']),
                'hostility': {},
            }

            hostility = result['hostility']

            # TODO(ayust): The fields returned by the API don't completely match up with
            # the fields available in-game. May want to revisit this in the future.

            standing = settings.find('onStandingDrop')
            hostility['standing'] = {
                'threshold': float(standing.attrib['standing']) / 100,
                'enabled': standing.attrib.get('enabled') != '0',
            }

            sec_status = settings.find('onStatusDrop')
            hostility['sec_status'] = {
                'threshold': float(sec_status.attrib['standing']) / 100,
                'enabled': sec_status.attrib.get('enabled') != '0',
            }

            hostility['aggression'] = {
                'enabled': settings.find('onAggression').get('enabled') != '0',
            }

            hostility['war'] = {
                'enabled': settings.find('onCorporationWar').get('enabled') != '0',
            }

            return result

        result = {
            'state': constants.Corp.pos_states[_int('state')],
            'state_ts': _ts('stateTimestamp'),
            'online_ts': _ts('onlineTimestamp'),
            'permissions': {
                'fuel': get_fuel_bay_perms(general_settings),
                'deploy': get_deploy_perms(general_settings),
                'forcefield': {
                    'corp': general_settings.find('allowCorporationMembers').text == '1',
                    'alliance': general_settings.find('allowAllianceMembers').text == '1',
                },
            },
            'combat': get_combat_settings(combat_settings),
            'fuel': {},
        }

        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            result['fuel'][int(a['typeID'])] = int(a['quantity'])

        return result

    def members(self, extended=True):
        """Returns details about each member of the corporation."""
        api_result = self.api.get('corp/MemberTracking', {'extended': int(extended)})

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            member = {
                'id': int(a['characterID']),
                'name': a['name'],
                'join_ts': api.parse_ts(a['startDateTime']),
                'base': {
                    # TODO(aiiane): Maybe remove this?
                    # It doesn't seem to ever have a useful value.
                    'id': int(a['baseID']),
                    'name': a['base'],
                },
                # Note that title does not include role titles,
                # only ones like 'CEO'
                'title': a['title'],
            }
            if extended:
                member.update({
                    'logon_ts': api.parse_ts(a['logonDateTime']),
                    'logoff_ts': api.parse_ts(a['logoffDateTime']),
                    'location': {
                        'id': int(a['locationID']),
                        'name': a['location'],
                    },
                    'ship_type': {
                        # "Not available" = -1 ship id; we change to None
                        'id': max(int(a['shipTypeID']), 0) or None,
                        'name': a['shipType'] or None,
                    },
                    'roles': int(a['roles']),
                    'can_grant': int(a['grantableRoles']),
                })

            results[member['id']] = member

        return results

    def permissions(self):
        """Returns information about corporation member permissions."""
        api_result = self.api.get('corp/MemberSecurity')

        results = {}
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            member = {
                'id': int(a['characterID']),
                'name': a['name'],
                'titles': {},
            }

            rowsets = dict((r.attrib['name'], r) for r in row.findall('rowset'))

            for title_row in rowsets['titles'].findall('row'):
                a = title_row.attrib
                member['titles'][int(a['titleID'])] = a['titleName']

            def get_roleset(roles_dict):
                roles_group = {}
                for key, rowset_name in roles_dict.iteritems():
                    roles = {}
                    roles_rowset = rowsets[rowset_name]
                    for role_row in roles_rowset.findall('row'):
                        a = role_row.attrib
                        roles[int(a['roleID'])] = a['roleName']
                    roles_group[key] = roles
                return roles_group

            member['roles'] = get_roleset(constants.Corp.role_types)
            member['can_grant'] = get_roleset(constants.Corp.grantable_types)

            results[member['id']] = member

        return results

    def permissions_log(self):
        """Returns information about changes to member permissions."""
        api_result = self.api.get('corp/MemberSecurityLog')

        inverse_role_types = dict((v,k) for k,v in constants.Corp.role_types.iteritems())

        results = []
        rowset = api_result.find('rowset')
        for row in rowset.findall('row'):
            a = row.attrib
            change = {
                'timestamp': api.parse_ts(a['changeTime']),
                'recipient': {
                    'id': int(a['characterID']),
                    'name': a['characterName'],
                },
                'issuer': {
                    'id': int(a['issuerID']),
                    'name': a['issuerName'],
                },
                'role_type': inverse_role_types[a['roleLocationType']],
                'roles': {
                    'before': {},
                    'after': {},
                },
            }

            rowsets = dict((r.attrib['name'], r) for r in row.findall('rowset'))
            old, new = change['roles']['before'], change['roles']['after']

            for role_row in rowsets['oldRoles'].findall('row'):
                a = role_row.attrib
                old[int(a['roleID'])] = a['roleName']

            for role_row in rowsets['newRoles'].findall('row'):
                a = role_row.attrib
                new[int(a['roleID'])] = a['roleName']

            results.append(change)

        results.sort(key=lambda r: r['timestamp'], reverse=True)
        return results

    def stations(self):
        """Returns information about the corporation's (non-POS) stations."""
        api_result = self.api.get('corp/OutpostList')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            station = {
                'id': int(a['stationID']),
                'owner_id': int(a['ownerID']),
                'name': a['stationName'],
                'system_id': int(a['solarSystemID']),
                'docking_fee_per_volume': float(a['dockingCostPerShipVolume']),
                'office_fee': int(a['officeRentalCost']),
                'type_id': int(a['stationTypeID']),
                'reprocessing': {
                    'efficiency': float(a['reprocessingEfficiency']),
                    'cut': float(a['reprocessingStationTake']),
                },
                'standing_owner_id': int(a['standingOwnerID']),
            }
            results[station['id']] = station

        return results

    def station_services(self, station_id):
        """Returns information about a given station's services."""
        api_result = self.api.get('corp/OutpostServiceDetail', {'itemID': station_id})

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            service = {
                'name': a['serviceName'],
                'owner_id': int(a['ownerID']),
                'standing': {
                    'minimum': float(a['minStanding']),
                    'bad_surcharge': float(a['surchargePerBadStanding']),
                    'good_discount': float(a['discountPerGoodStanding']),
                },
            }
            results[service['name']] = service

        return results

    def medals(self):
        """Returns information about the medals created by a corporation."""
        api_result = self.api.get('corp/Medals')

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            medal = {
                'id': int(a['medalID']),
                'creator_id': int(a['creatorID']),
                'title': a['title'],
                'description': a['description'],
                'create_ts': api.parse_ts(a['created']),
            }
            results[medal['id']] = medal

        return results

    def member_medals(self):
        """Returns information about medals assigned to corporation members."""
        api_result = self.api.get("corp/MemberMedals")

        rowset = api_result.find('rowset')
        results = {}
        for row in rowset.findall('row'):
            a = row.attrib
            award = {
                'medal_id': int(a['medalID']),
                'char_id': int(a['characterID']),
                'reason': a['reason'],
                'public': a['status'] == 'public',
                'issuer_id': int(a['issuerID']),
                'timestamp': api.parse_ts(a['issued']),
            }
            results.setdefault(award['char_id'], {})[award['medal_id']] = award

        return results

    def container_log(self):
        """Returns a log of actions performed on corporation containers."""
        api_result = self.api.get("corp/ContainerLog")

        results = []
        rowset = api_result.find('rowset')

        def int_or_none(val):
            return int(val) if val else None

        for row in rowset.findall('row'):
            a = row.attrib
            action = {
                'timestamp': api.parse_ts(a['logTime']),
                'item': {
                    'id': int(a['itemID']),
                    'type_id': int(a['itemTypeID']),
                },
                'actor': {
                    'id': int(a['actorID']),
                    'name': a['actorName'],
                },
                'location_id': int(a['locationID']),
                'action': a['action'],
                'details': {
                    # TODO(aiiane): Find a translation for this flag field
                    'flag': int(a['flag']),
                    'password_type': a['passwordType'] or None,
                    'type_id': int_or_none(a['typeID']),
                    'quantity': int_or_none(a['quantity']),
                    'config': {
                        'old': int_or_none(a['oldConfiguration']),
                        'new': int_or_none(a['newConfiguration']),
                    },
                },
            }
            results.append(action)

        return results


# vim: set ts=4 sts=4 sw=4 et:



import shelve

from evelink import api

class ShelveCache(api.APICache):
    """An implementation of APICache using shelve."""

    def __init__(self, path):
        super(ShelveCache, self).__init__()
        self.cache = shelve.open(path)



from evelink import api

def parse_kills(api_result):
    rowset = api_result.find('rowset')
    result = {}
    for row in rowset.findall('row'):
        a = row.attrib
        kill_id = int(a['killID'])
        result[kill_id] = {
            'id': kill_id,
            'system_id': int(a['solarSystemID']),
            'time': api.parse_ts(a['killTime']),
            'moon_id': int(a['moonID']),
        }

        victim = row.find('victim')
        a = victim.attrib
        result[kill_id]['victim'] = {
            'id': int(a['characterID']),
            'name': a['characterName'],
            'corp': {
                'id': int(a['corporationID']),
                'name': a['corporationName'],
            },
            'alliance': {
                'id': int(a['allianceID']),
                'name': a['allianceName'],
            },
            'faction': {
                'id': int(a['factionID']),
                'name': a['factionName'],
            },
            'damage': int(a['damageTaken']),
            'ship_type_id': int(a['shipTypeID']),
        }

        result[kill_id]['attackers'] = {}

        rowsets = {}
        for rowset in row.findall('rowset'):
            key = rowset.attrib['name']
            rowsets[key] = rowset

        for attacker in rowsets['attackers'].findall('row'):
            a = attacker.attrib
            attacker_id = int(a['characterID'])
            result[kill_id]['attackers'][attacker_id] = {
                'id': attacker_id,
                'name': a['characterName'],
                'corp': {
                    'id': int(a['corporationID']),
                    'name': a['corporationName'],
                },
                'alliance': {
                    'id': int(a['allianceID']),
                    'name': a['allianceName'],
                },
                'faction': {
                    'id': int(a['factionID']),
                    'name': a['factionName'],
                },
                'sec_status': float(a['securityStatus']),
                'damage': int(a['damageDone']),
                'final_blow': a['finalBlow'] == '1',
                'weapon_type_id': int(a['weaponTypeID']),
                'ship_type_id': int(a['shipTypeID']),
            }

        result[kill_id]['items'] = {}
        for item in rowsets['items'].findall('row'):
            a = item.attrib
            type_id = int(a['typeID'])
            result[kill_id]['items'][type_id] = {
                'id': type_id,
                'flag': int(a['flag']),
                'dropped': int(a['qtyDropped']),
                'destroyed': int(a['qtyDestroyed']),
            }

    return result



import pickle
import time
import sqlite3

from evelink import api

class SqliteCache(api.APICache):
    """An implementation of APICache using sqlite."""

    def __init__(self, path):
        super(SqliteCache, self).__init__()
        self.connection = sqlite3.connect(path)
        cursor = self.connection.cursor()
        cursor.execute('create table if not exists cache ("key" text primary key on conflict replace,'
                       'value blob, expiration integer)')

    def get(self, key):
        cursor = self.connection.cursor()
        cursor.execute('select value, expiration from cache where "key"=?',(key,))
        result = cursor.fetchone()
        if not result:
            return None
        value, expiration = result
        if expiration < time.time():
            cursor.execute('delete from cache where "key"=?', (key,))
            self.connection.commit()
            return None
        cursor.close()
        return pickle.loads(str(value))

    def put(self, key, value, duration):
        expiration = time.time() + duration
        value_tuple = (key, sqlite3.Binary(pickle.dumps(value, 2)), expiration)
        cursor = self.connection.cursor()
        cursor.execute('insert into cache values (?, ?, ?)', value_tuple)
        self.connection.commit()
        cursor.close()
